################################################################################
# Feuerfuchs Addons Installer
#   file: Main.nsi
#   desc: The `Main' structure. This file should be compiled!
#   author: hobbyscripter<hobbyscripter@gmail.com>
#   version: 2.0.1.0
#   copyright: abandoned!
################################################################################



################################################################################
# Compressor
################################################################################
SetCompressor LZMA



################################################################################
# Include
################################################################################

; User Config
!include "UserConfig.nsh"

; Locale
!include "Source\LangTab\__LocaleInclude.nsh"

; NSIS Library
!include "MUI.nsh"
!include "LogicLib.nsh"
!include "WinMessages.nsh"
!include "Sections.nsh"

; Global Defination
!include "Source\Global.nsh"

; Compile-Time Custom Library
!include "Source\Lib\Counter.nsh"
!include "Source\Lib\FindProc.nsh"
!include "Source\Lib\AdvSectionsLib.nsh"
!include "Source\Lib\MessageBoxLib.nsh"

; Addons Class
!include "Source\AddonsClass\Extension.nsh"
!include "Source\AddonsClass\Theme.nsh"
!include "Source\AddonsClass\SearchPlugin.nsh"
!include "Source\AddonsClass\Group.nsh"

; Addons List
!include "AddonsList.nsh"

; GUI Library
!include "Source\WidgetsClass\General.nsh"
!include "Source\WidgetsClass\ListBox.nsh"
!include "Source\WidgetsClass\RadioBox.nsh"

; Runtime Custom Library
!include "Source\Lib\TargetAppHandler.nsh"
!include "Source\Lib\ProfileHandler.nsh"

; Pages
!include "Source\PageClass\ProfileSettingPage.nsh"
!include "Source\PageClass\TargetAppSettingPage.nsh"
!include "Source\PageClass\ComponentsPage.nsh"
!include "Source\PageClass\CloseTargetAppPage.nsh"
!include "Source\PageClass\ExtensionInstallHintPage.nsh"



################################################################################
# Program Basic Information
################################################################################
Name "${NAME}"
OutFile "${OUTPUT_FILEPATH}"
BrandingText "${BRANDING_TEXT}"
XPStyle on
RequestExecutionLevel admin



################################################################################
# Version Information
################################################################################
VIProductVersion "${VERSION}"
VIAddVersionKey "ProductName" "${VI_PRODUCT_NAME}"
VIAddVersionKey "Comments" "${VI_COMMENTS}"
VIAddVersionKey "CompanyName" "${VI_COMPANY_NAME}"
VIAddVersionKey "LegalCopyright" "${VI_LEGAL_COPYRIGHT}"
VIAddVersionKey "FileDescription" "${VI_FILE_DESC}"
VIAddVersionKey "FileVersion" "${VI_FILE_VERSION}"
VIAddVersionKey "ProductVersion" "${VI_PRODUCT_VERSION}"
VIAddVersionKey "LegalTrademarks" "${VI_LEGAL_TRADEMARKS}"



################################################################################
# Installer Resource
################################################################################
ReserveFile "Source\Resources\FormLayout\ProfileSettingPage.ini"
ReserveFile "Source\Resources\FormLayout\TargetAppSettingPage.ini"
ReserveFile "Source\Resources\FormLayout\CloseTargetAppPage.ini"
ReserveFile "Source\Resources\Image\CloseTargetAppImage.bmp"
ReserveFile "Source\Resources\Image\ExtensionInstall.bmp"
ReserveFile "AddonsPack\RelationTab.ini"



################################################################################
# User Interface
################################################################################
; Icon
!if "${ICON_PATH}" != ""
    !define MUI_ICON "${ICON_PATH}"
!else
    !define MUI_ICON "${NSISDIR}\Contrib\Graphics\Icons\orange-install.ico"
!endif

; Welcome/Finish Left Image
!if "${LEFT_IMAGE_PATH}" != ""
    !define MUI_WELCOMEFINISHPAGE_BITMAP "${LEFT_IMAGE_PATH}"
!else
    !define MUI_WELCOMEFINISHPAGE_BITMAP "${NSISDIR}\Contrib\Graphics\Wizard\orange.bmp"
!endif

; Welcome Words
!if "${WELCOME_WORDS}" != ""
    !define MUI_WELCOMEPAGE_TEXT "${WELCOME_WORDS}"
!endif

; Show Welcome Page
!insertmacro MUI_PAGE_WELCOME

; Show License Page
!if "${LICENSE_FOR_INSTALLER}" != ""
    !insertmacro MUI_PAGE_LICENSE "${LICENSE_FOR_INSTALLER}"
!endif
!if "${LICENSE_FOR_ADDONS}" != ""
    !insertmacro MUI_PAGE_LICENSE "${LICENSE_FOR_ADDONS}"
!endif

; Profile Selector
Page custom ProfileSettingPageInit ProfileSettingPageCallback

; Find TargetApp Executables Page
Page custom TargetAppSettingPageInit TargetAppSettingPageCallback

; Components Selecting Page
!define MUI_COMPONENTSPAGE_SMALLDESC
!define MUI_PAGE_CUSTOMFUNCTION_PRE ComponentsPageInit
!insertmacro MUI_PAGE_COMPONENTS

; Close TargetApp Page
Page custom CloseTargetAppPageInit CloseTargetAppPageCallback

; Extension Install Hint Page
!if "${PROVIDE_APP_INSTALLER}" == "true"
Page custom ExtensionInstallHintPageInit1 ; For Install TargetApp User
!endif

; Start Installing Files
!insertmacro MUI_PAGE_INSTFILES

; Extension Install Hint Page
Page custom ExtensionInstallHintPageInit2 ; For Other User

; Languague of Installer
!include "${__MUI_LOCALE_LIST}"
!delfile "${__MUI_LOCALE_LIST}"



################################################################################
# OnInit Function
################################################################################
Function .onInit
    ; ----- Show Locale Dialog -----
    !if "${SHOW_SELECT_LANG_DIALOG}" == "true"
    !insertmacro SelectLanguageDialog
    !endif

    ; ----- Reserves the Files for Install Options -----
    !insertmacro MUI_INSTALLOPTIONS_EXTRACT_AS "Source\Resources\FormLayout\ProfileSettingPage.ini" "ProfileSettingPage.ini"
    !insertmacro MUI_INSTALLOPTIONS_EXTRACT_AS "Source\Resources\FormLayout\TargetAppSettingPage.ini" "TargetAppSettingPage.ini"
    !insertmacro MUI_INSTALLOPTIONS_EXTRACT_AS "Source\Resources\FormLayout\CloseTargetAppPage.ini" "CloseTargetAppPage.ini"
    !insertmacro MUI_INSTALLOPTIONS_EXTRACT_AS "Source\Resources\FormLayout\ExtensionInstallHintPage.ini" "ExtensionInstallHintPage.ini"

    ; ----- Reserve Files for Advanced Sections Library -----
    Call ExtractAdvSectionsLibFile

    ; ----- Extract Files -----
    File "/oname=${RELATION_TAB}" "AddonsPack\RelationTab.ini"
FunctionEnd



################################################################################
# Install Components
################################################################################

# ---------------
# TargetApp Installer Section
# ---------------
!if "${PROVIDE_APP_INSTALLER}" == "true"
Section /o "$(SectionInstallTargetApp)" "Section_InstallApp"
    File "/oname=$PLUGINSDIR\appInstaller.exe" "${APP_INSTALLER_PATH}" ; Extract the installer to $PLUGINSDIR

    ; Installer will be executed at `App Installer Section (II)'
SectionEnd
${RECORD_SECTION_INDEX} "Section_InstallApp"
!endif


# ---------------
# Extension Section Group
# ---------------
!if ${EXTENSION_COUNTER} > 0
SectionGroup /e "$(SectionGroupExtensions)" "SectionGroup_Extension"
    !include ${__EXTENSIONS_SECTION}
    !delfile ${__EXTENSIONS_SECTION}
SectionGroupEnd
!endif


# ---------------
# Theme Section Group
# ---------------
!if ${THEME_COUNTER} > 0
SectionGroup "$(SectionGroupThemes)" "SectionGroup_Theme"
    !include ${__THEMES_SECTION}
    !delfile ${__THEMES_SECTION}
SectionGroupEnd
!endif


# ---------------
# Search Plugin Section Group
# ---------------
!if ${SEARCHPLUGIN_COUNTER} > 0
SectionGroup "$(SectionGroupSearchPlugins)" "SectionGroup_SearchPlugin"
    !include ${__SEARCHPLUGINS_SECTION}
    !delfile ${__SEARCHPLUGINS_SECTION}
SectionGroupEnd
!endif


# ---------------
# TargetApp Installer Section (II)
# ---------------
!if "${PROVIDE_APP_INSTALLER}" == "true"
Section "-AppInstallerExecute"
    ; ----- Variables Define -----
    Push $0
    !define RECALL_OR_NOT $0

    ; ----- Main -----
    ${If} ${SectionIsSelectedAdv} "Section_InstallApp"
        ${DoWhile} "$_TargetAppExecutable_" == ""
            ExecWait "$PLUGINSDIR\appInstaller.exe"

            ${FindTargetAppEXE} ; Update TargetApp Information
            ${If} ${Errors}
                ClearErrors
                ${MBYesNoValue} ${RECALL_OR_NOT} "$(QRestartTargetAppInstaller)"

                ${If} "${RECALL_OR_NOT}" == "no"
                    ${Break}
                ${EndIf}
            ${EndIf}
        ${Loop}
    ${EndIf}


    ; ----- Variables Undefine -----
    !undef RECALL_OR_NOT
    Pop $0
SectionEnd
!endif


# ---------------
# Create Profile Manager Link
# ---------------
Section "-CreateProfileManagerLink"
    ; ----- Variables Define -----
    Push $0
    !define CREATE_OR_NOT $0

    ; ----- Main -----
    ${If} "$_ProfileType_" != "Browse"
        ${If} "$_AppEXEType_" != "Ignore"
            ${MBYesNoValue} ${CREATE_OR_NOT} "$(QCreateShortcutForProfileManager)"

            ${If} ${CREATE_OR_NOT} == "yes"
                SetShellVarContext current

                ${If} ${FileExists} "${TARGETAPP_SHORTCUT_DIR}"
                    ${If} ${FileExists} "${PROFILE_MANAGER_SHORTCUT}"
                        Delete "${PROFILE_MANAGER_SHORTCUT}"
                    ${EndIf}

                    CreateShortCut "${PROFILE_MANAGER_SHORTCUT}" "$_TargetAppExecutable_" "-profilemanager"
                ${Else}
                    SetShellVarContext all

                    ${If} ${FileExists} "${TARGETAPP_SHORTCUT_DIR}"
                        ${If} ${FileExists} "${PROFILE_MANAGER_SHORTCUT}"
                            Delete "${PROFILE_MANAGER_SHORTCUT}"
                        ${EndIf}

                        CreateShortCut "${PROFILE_MANAGER_SHORTCUT}" "$_TargetAppExecutable_" "-profilemanager"
                    ${EndIf}
                ${EndIf}
            ${EndIf}
        ${EndIf}
    ${EndIf}

    ; ----- Variables Undefine -----
    !undef CREATE_OR_NOT
    Pop $0
SectionEnd



################################################################################
# Functions
################################################################################

# ---------------
# Deal with `AdvSectionsLib' Data
# ---------------
Function ExtractAdvSectionsLibFile
    ReserveFile "${__SECTIONS_IDX_TAB}"
    ReserveFile "${__SECTIONS_STATE_TAB}"
    File "/oname=${SECTIONS_IDX_TAB}" "${__SECTIONS_IDX_TAB}"
    File "/oname=${SECTIONS_STATE_TAB}" "${__SECTIONS_STATE_TAB}"
    !delfile "${__SECTIONS_IDX_TAB}"
    !delfile "${__SECTIONS_STATE_TAB}"
FunctionEnd

# ---------------
# After Successful Installation
# ---------------
Function .onInstSuccess
    ; ----- Variables Define -----
    Push $0
    !define EXEC_OR_NOT $0

    ; ----- Main -----
    ${If} "$_AppEXEType_" != "Ignore"
        ${MBYesNoValue} ${EXEC_OR_NOT} "$(QLaunchTargetApp)"

        ${If} "${EXEC_OR_NOT}" == "yes"
            Exec '"$_TargetAppExecutable_" -profile "$_TargetProfileDir_"'
        ${EndIf}
    ${EndIf}

    ; ----- Variables Undefine -----
    !undef EXEC_OR_NOT
    Pop $0
FunctionEnd



################################################################################
# Description Message
################################################################################
!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
    !if "${PROVIDE_APP_INSTALLER}" == "true"
    !insertmacro MUI_DESCRIPTION_TEXT ${Section_InstallApp} "$(SectionInstallTargetApp)"
    !endif

    !if ${EXTENSION_COUNTER} > 0
    !insertmacro MUI_DESCRIPTION_TEXT ${SectionGroup_Extension} "$(SectionGroupExtensions)"
    !endif

    !if ${THEME_COUNTER} > 0
    !insertmacro MUI_DESCRIPTION_TEXT ${SectionGroup_Theme} "$(SectionGroupThemes)"
    !endif

    !if ${SEARCHPLUGIN_COUNTER} > 0
    !insertmacro MUI_DESCRIPTION_TEXT ${SectionGroup_SearchPlugin} "$(SectionGroupSearchPlugins)"
    !endif

    ; Description of Extensions
    !include "${__EXTENSIONS_DESC}"
    !delfile "${__EXTENSIONS_DESC}"

    ; Description of Themes
    !include "${__THEMES_DESC}"
    !delfile "${__THEMES_DESC}"

    ; Description of SearchPlugins
    !include "${__SEARCHPLUGINS_DESC}"
    !delfile "${__SEARCHPLUGINS_DESC}"

    ; Description of SectionGroup
    !include "${__SECTIONGROUPS_DESC}"
    !delfile "${__SECTIONGROUPS_DESC}"
!insertmacro MUI_FUNCTION_DESCRIPTION_END




