################################################################################
# Feuerfuchs Addons Installer -- Advanced Sections Library
#   file: Source\Lib\AdvSectionsLib.nsh
#   desc: To provide advance sections controling features.
#   author: hobbyscripter<hobbyscripter@gmail.com>
#   version: 2.0.1.0
#   copyright: abandoned!
################################################################################



################################################################################
# Temp Variables
################################################################################
Var "_AdvSecTempSectionIdx_"
Var "_AdvSecTempSectionState_"
Var "_AdvSecTempSectionRecordState_"
Var "_AdvSecTempSectionSelected_"



################################################################################
# Macros
################################################################################

# ---------------
# Temp File Handler
# ---------------
!tempfile __SECTIONS_IDX_TAB ; One for SectionIdx(Name)-SectionIdx(Digital) Cross Table
!tempfile __SECTIONS_STATE_TAB ; One for Section Status

!define __WRITE_SECTIONS_IDX_TAB '!appendfile "${__SECTIONS_IDX_TAB}"'
!define __WRITE_SECTIONS_STATE_TAB '!appendfile "${__SECTIONS_STATE_TAB}"'


# ---------------
# Record Section Index ; here comes SectionIndexName-SectionIndex table
# ---------------
!macro RECORD_SECTION_INDEX SECIDNAME
    !ifndef __SECTIONS_IDX_TAB_HEADER
        !define __SECTIONS_IDX_TAB_HEADER
        ; Write Section Index Tab Header
        ${__WRITE_SECTIONS_IDX_TAB} '[Data]$\n'
    !endif

    ${__WRITE_SECTIONS_IDX_TAB} '${SECIDNAME}=${${SECIDNAME}}$\n'
!macroend
!define RECORD_SECTION_INDEX "!insertmacro RECORD_SECTION_INDEX"


# ---------------
# Record Section Default Select State
# ---------------
!macro RECORD_SECTION_STATE SECIDNAME SELSTATE
    !ifndef __SECTIONS_STATE_TAB_HEADER
        !define __SECTIONS_STATE_TAB_HEADER
        ; Write Section State Tab Header
        ${__WRITE_SECTIONS_STATE_TAB} '[Data]$\n'
    !endif

    !if "${SELSTATE}" == "true"
        ; Selected in Default
        ${__WRITE_SECTIONS_STATE_TAB} '${SECIDNAME}=1$\n'
    !else
        ; Unselected in Default
        ${__WRITE_SECTIONS_STATE_TAB} '${SECIDNAME}=0$\n'
    !endif
!macroend
!define RECORD_SECTION_STATE "!insertmacro RECORD_SECTION_STATE"


# ---------------
# Update Section STATE
# ---------------
!macro UPDATE_SECTION_STATE SECIDNAME
    ReadINIStr $_AdvSecTempSectionIdx_ "${SECTIONS_IDX_TAB}" "Data" "${SECIDNAME}"

    ${If} ${SectionIsSelected} $_AdvSecTempSectionIdx_
        WriteINIStr "${SECTIONS_STATE_TAB}" "Data" "${SECIDNAME}" "1"
    ${Else}
        WriteINIStr "${SECTIONS_STATE_TAB}" "Data" "${SECIDNAME}" "0"
    ${EndIf}
!macroend
!define UPDATE_SECTION_STATE "!insertmacro UPDATE_SECTION_STATE"



# ---------------
# Section Is Selected Advanced
# ---------------
!macro _SectionIsSelectedAdv _a _b _t _f
    ReadINIStr $_AdvSecTempSectionIdx_ "${SECTIONS_IDX_TAB}" "Data" "${_b}"
    !insertmacro _SectionFlagIsSet "${SF_SELECTED}" "$_AdvSecTempSectionIdx_" "${_t}" "${_f}"
!macroend
!define SectionIsSelectedAdv `"" SectionIsSelectedAdv`



# ---------------
# Section Is Modified
# ---------------
!macro _SectionIsModified _a _b _t _f
    ReadINIStr $_AdvSecTempSectionIdx_ "${SECTIONS_IDX_TAB}" "Data" "${_b}"
    ReadINIStr $_AdvSecTempSectionRecordState_ "${SECTIONS_STATE_TAB}" "Data" "${_b}"

    SectionGetFlags $_AdvSecTempSectionIdx_ $_AdvSecTempSectionState_
    IntOp $_AdvSecTempSectionSelected_ $_AdvSecTempSectionState_ & ${SF_SELECTED}


      !define IfTrueLabel "IfTrue_${__LINE__}"
      !define IfFalseLabel "IfFalse_${__LINE__}"
      !define EndIfLabel "EndIf_${__LINE__}"

    IntCmp $_AdvSecTempSectionRecordState_ 1 ${IfTrueLabel} ${IfFalseLabel} ${IfFalseLabel}
        ${IfTrueLabel}:
            ; Is Selected In Record
            !insertmacro _SectionFlagIsSet "${SF_SELECTED}" "$_AdvSecTempSectionIdx_" "${_f}" "${_t}" ; Do Nothing if Selected
            Goto ${EndIfLabel}
        ${IfFalseLabel}:
            !insertmacro _SectionFlagIsSet "${SF_SELECTED}" "$_AdvSecTempSectionIdx_" "${_t}" "${_f}" ; Do Nothing if Unselected
            Goto ${EndIfLabel}
    ${EndIfLabel}:

      !undef IfTrueLabel
      !undef IfFalseLabel
      !undef EndIfLabel

!macroend
!define SectionIsModified `"" SectionIsModified`


# ---------------
# Get Section Name by Section Index Name (Macro)
# ---------------
!macro GetSectionNameBySecIdxName RETURNVAL SECIDNAME
    Push "${SECIDNAME}"
    Exch $0 ; $0 = ${SECIDNAME}, STACK = $0
    Push $1 ; STACK = $0|$1
    Push $2 ; STACK = $0|$1|$2
    Call GetSectionNameBySecIdxName
    Exch $2 ; STACK = $0|$1|[output]
    Exch 2 ; STACK = [output]|$1|$0
    Pop $0 ; STACK = [output]|$1
    Pop $1 ; STACK = [output]
    Pop ${RETURNVAL}
!macroend
!define GetSectionNameBySecIdxName "!insertmacro GetSectionNameBySecIdxName"


# ---------------
# Select Section Adv
# ---------------
!macro SelectSectionAdv SECIDNAME
    ReadINIStr $_AdvSecTempSectionIdx_ "${SECTIONS_IDX_TAB}" "Data" "${SECIDNAME}"
    !insertmacro SelectSection $_AdvSecTempSectionIdx_
!macroend
!define SelectSectionAdv "!insertmacro SelectSectionAdv"


# ---------------
# Unselect Section Adv
# ---------------
!macro UnselectSectionAdv SECIDNAME
    ReadINIStr $_AdvSecTempSectionIdx_ "${SECTIONS_IDX_TAB}" "Data" "${SECIDNAME}"
    !insertmacro UnselectSection $_AdvSecTempSectionIdx_
!macroend
!define UnselectSectionAdv "!insertmacro UnselectSectionAdv"



################################################################################
# Functions
################################################################################

# ---------------
# Get Section Name by Section Index Name (Macro)
# ---------------
Function GetSectionNameBySecIdxName
    ; ----- Variables Define -----
    !define SECIDNAME $0
    !define SECID $1
    !define SECNAME $2

    ; ----- Main -----
    ReadINIStr ${SECID} "${SECTIONS_IDX_TAB}" "Data" "${SECIDNAME}"
    SectionGetText ${SECID} ${SECNAME}

    ; ----- Variables Undefine -----
    !undef SECIDNAME
    !undef SECID
    !undef SECNAME
FunctionEnd




