################################################################################
# Feuerfuchs Addons Installer -- Profile Handler
#   file: Source\Lib\ProfileHandler.nsh
#   desc: To deal with existing profile or not existing profile
#   author: hobbyscripter<hobbyscripter@gmail.com>
#   version: 2.0.1.0
#   copyright: abandoned!
################################################################################



################################################################################
# Variables
################################################################################
Var "_DefaultProfileId_"



################################################################################
# Macro
################################################################################

# ---------------
# Read Profile INI (Macro)
# ---------------
!macro ReadProfilesINI
    Push $0
    Push $1
    Push $2
    Push $3
    Push $4
    Call ReadProfilesINI
    Pop $4
    Pop $3
    Pop $2
    Pop $1
    Pop $0
!macroend
!define ReadProfilesINI "!insertmacro ReadProfilesINI"

# ---------------
# Create A Profile
# ---------------
!macro CreateNewProfile PROFILE_NAME
    Push ${PROFILE_NAME}
    Exch $0 ; $0 = TARGET_PROFILE_NAME
    Push $1
    Push $2
    Push $3
    Push $4
    Call CreateNewProfile
    Pop $4
    Pop $3
    Pop $2
    Pop $1
    Pop $0
!macroend
!define CreateNewProfile "!insertmacro CreateNewProfile"



################################################################################
# Functions
################################################################################

# ---------------
# Read Profile INI
# ---------------
Function ReadProfilesINI
    ; ----- Define Variables Name -----
    !define PROFILE_NAME $0
    !define PROFILE_DEFAULT_VALUE $1
    !define PROFILE_PATH $2
    !define PROFILE_IS_RELATIVE $3
    !define COUNT $4

    ; ----- Main -----

    ; Check the Existence
    ${Unless} ${FileExists} "${PROFILES_INI_PATH}"
        ; profiles.ini doesn't exists
        IntOp $_ExistProfilesCount_ 0 + 0
        Return
    ${EndUnless}

    ; Delete Existing Profile Tab
    ${If} ${FileExists} "$PLUGINSDIR\ProfileTab.ini"
        Delete "$PLUGINSDIR\ProfileTab.ini"
    ${EndIf}

    ; List All the Profile Name
    IntOp ${COUNT} 0 + 0
    ClearErrors
    ${Do}
        ReadINIStr ${PROFILE_NAME} ${PROFILES_INI_PATH} "Profile${COUNT}" "Name"

        ; Check If This is End
        ${If} ${Errors}
            ; End of profiles.ini
            ClearErrors
            ${ExitDo}
        ${EndIf}

        ; Read Profile Path
        ReadINIStr ${PROFILE_PATH} "${PROFILES_INI_PATH}" "Profile${COUNT}" "Path"
        ReadINIStr ${PROFILE_IS_RELATIVE} "${PROFILES_INI_PATH}" "Profile${COUNT}" "IsRelative"

        ${If} ${PROFILE_IS_RELATIVE} == "1"
            GetFullPathName ${PROFILE_PATH} "${PROFILES_MANAGER_DIR}\${PROFILE_PATH}"
        ${EndIf}

        ; Check If This is Default
        ReadINIStr ${PROFILE_DEFAULT_VALUE} "${PROFILES_INI_PATH}" "Profile${COUNT}" "Default"
        ${If} ${Errors}
            ClearErrors
        ${Else}
            ${If} ${PROFILE_DEFAULT_VALUE} == "1"
                StrCpy $_DefaultProfileId_ "${COUNT}"
            ${EndIf}
        ${EndIf}

        ; Make My Table
        WriteINIStr "$PLUGINSDIR\ProfileTab.ini" "NameTab" "Name${COUNT}" "${PROFILE_NAME}"
        WriteINIStr "$PLUGINSDIR\ProfileTab.ini" "PathTab" "Path${COUNT}" "${PROFILE_PATH}"

        IntOp ${COUNT} ${COUNT} + 1
    ${Loop}

    ; Count of Profiles
    IntOp $_ExistProfilesCount_ ${Count} + 0

    ; ----- Undefine Variable Name -----
    !undef PROFILE_NAME
    !undef PROFILE_DEFAULT_VALUE
    !undef PROFILE_PATH
    !undef PROFILE_IS_RELATIVE
    !undef COUNT
FunctionEnd


# ---------------
# Create A Profile
# ---------------
Function CreateNewProfile
    ; ----- Define Variables Name -----
    !define PROFILE_NAME $0
    !define COUNT_NUMBER $1
    !define RANDOM_WORD $2
    !define PROFILES_NUM $3
    !define COUNT $4

    ; ------ Main -----

    ; Initialize Random Word
    IntOp ${COUNT_NUMBER} 0 + 0
    IntFmt ${RANDOM_WORD} "%08X" ${COUNT_NUMBER}

    ; Find the Directory
    ${While} ${FileExists} "${PROFILES_MANAGER_DIR}\profiles\${RANDOM_WORD}.${PROFILE_NAME}"
        ${If} ${COUNT_NUMBER} = 2147483646
            MessageBox MB_OK|MB_ICONEXCLAMATION "$(CantCreateProfileByThisName)"
            SetErrors
            Return
        ${EndIf}

        IntOp ${COUNT_NUMBER} ${COUNT_NUMBER} + 1 ; Try Next Number
        IntFmt ${RANDOM_WORD} "%08X" ${COUNT_NUMBER}
    ${EndWhile}

    ; Create It
    CreateDirectory "${PROFILES_MANAGER_DIR}\profiles\${RANDOM_WORD}.${PROFILE_NAME}"

    ; Create plugins.ini
    ${If} $_ExistProfilesCount_ == 0
        WriteINIStr "${PROFILES_INI_PATH}" "General" "StartWithLastProfile" "1"
        WriteINIStr "${PROFILES_INI_PATH}" "Profile0" "Name" "${PROFILE_NAME}"
        WriteINIStr "${PROFILES_INI_PATH}" "Profile0" "IsRelative" "1"
        WriteINIStr "${PROFILES_INI_PATH}" "Profile0" "Path" "Profiles/${RANDOM_WORD}.${PROFILE_NAME}"
    ${Else}
        WriteINIStr "${PROFILES_INI_PATH}" "Profile$_ExistProfilesCount_" "Name" "${PROFILE_NAME}"
        WriteINIStr "${PROFILES_INI_PATH}" "Profile$_ExistProfilesCount_" "IsRelative" "1"
        WriteINIStr "${PROFILES_INI_PATH}" "Profile$_ExistProfilesCount_" "Path" "Profiles/${RANDOM_WORD}.${PROFILE_NAME}"
    ${EndIf}


    ; Changes the Default
    ${For} ${COUNT} 0 $_ExistProfilesCount_
        ; Clear Old Setting
        WriteINIStr "${PROFILES_INI_PATH}" "Profile${COUNT}" "Default" "0"
    ${Next}
    WriteINIStr "${PROFILES_INI_PATH}" "Profile$_ExistProfilesCount_" "Default" "1" ; Overwrite

    ; Changes the Profiles Count
    IntOp $_ExistProfilesCount_ $_ExistProfilesCount_ + 1

    ; Assign the Created Profile Path to $_TargetProfileDir_
    StrCpy $_TargetProfileDir_ "${PROFILES_MANAGER_DIR}\profiles\${RANDOM_WORD}.${PROFILE_NAME}"

    ; ----- Undefine Variables Name -----
    !undef PROFILES_NUM
    !undef PROFILE_NAME
    !undef COUNT_NUMBER
    !undef RANDOM_WORD
    !undef COUNT
FunctionEnd




